function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

/**
 * Cede control to the event loop for one tick, from within an async function.
 *
 * Usage:
 *
 *     // ... slow stuff...
 *     await tick() // will cede control
 *     // ... slow stuff...
 */
function tick() {
  return new Promise(function (resolve) {
    return setTimeout(resolve, 0);
  });
}

var KeyClusterer =
/*#__PURE__*/
function () {
  function KeyClusterer(bucket, keyer, options) {
    _classCallCheck(this, KeyClusterer);

    this.bucket = bucket;
    this.keyer = keyer;
    this.options = options;
    this.progress = 0;
    this.canceled = false;
  }

  _createClass(KeyClusterer, [{
    key: "cancel",
    value: function cancel() {
      this.canceled = true;
    }
  }, {
    key: "cluster",
    value: function () {
      var _cluster = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee() {
        var bucket, keyer, _this$options, tickMs, nIterationsBetweenTickChecks, bins, keyToBin, i, t1, strs, _i, str, t2, count, key, bin, maxCount;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                bucket = this.bucket, keyer = this.keyer;
                _this$options = this.options, tickMs = _this$options.tickMs, nIterationsBetweenTickChecks = _this$options.nIterationsBetweenTickChecks;
                bins = [];
                keyToBin = {};
                i = 0;
                t1 = new Date();
                strs = Object.keys(bucket);
                _i = 0;

              case 8:
                if (!(_i < strs.length)) {
                  _context.next = 29;
                  break;
                }

                str = strs[_i];
                i += 1;

                if (!((i & nIterationsBetweenTickChecks) === 0)) {
                  _context.next = 20;
                  break;
                }

                t2 = new Date();

                if (!(t2 - t1 >= tickMs)) {
                  _context.next = 20;
                  break;
                }

                this.progress = (i - 1) / strs.length;
                _context.next = 17;
                return tick();

              case 17:
                if (!this.canceled) {
                  _context.next = 19;
                  break;
                }

                throw new Error('canceled');

              case 19:
                t1 = new Date();

              case 20:
                count = bucket[str];
                key = keyer(str);
                bin = keyToBin[key];

                if (!bin) {
                  bin = {
                    key: key,
                    name: str,
                    count: 0,
                    bucket: {}
                  };
                  keyToBin[key] = bin;
                  bins.push(bin);
                } else {
                  // Maybe change name. We do it in this loop so we're O(n)
                  maxCount = bin.bucket[bin.name];

                  if (count > maxCount || count === maxCount && str.localeCompare(bin.name) < 0) {
                    bin.name = str;
                  }
                }

                bin.count += count;
                bin.bucket[str] = count;

              case 26:
                _i++;
                _context.next = 8;
                break;

              case 29:
                this.progress = 1;
                return _context.abrupt("return", bins.filter(function (b) {
                  return Object.keys(b.bucket).length > 1;
                }));

              case 31:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function cluster() {
        return _cluster.apply(this, arguments);
      };
    }()
  }]);

  return KeyClusterer;
}();

function clusterByKey(bucket, keyer) {
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  options = _objectSpread({
    tickMs: 8,
    nIterationsBetweenTickChecks: 0xfff
  }, options);
  return new KeyClusterer(bucket, keyer, options);
}

var KnnClusterer =
/*#__PURE__*/
function () {
  function KnnClusterer(bucket, distance, radius, options) {
    _classCallCheck(this, KnnClusterer);

    this.bucket = bucket;
    this.distance = distance;
    this.radius = radius;
    this.options = options;
    this.progress = 0;
    this.canceled = false;
  }

  _createClass(KnnClusterer, [{
    key: "cancel",
    value: function cancel() {
      this.canceled = true;
    }
  }, {
    key: "cluster",
    value: function () {
      var _cluster = _asyncToGenerator(
      /*#__PURE__*/
      regeneratorRuntime.mark(function _callee() {
        var bucket, distance, radius, _this$options, tickMs, nIterationsBetweenTickChecks, strs, bins, strToBin, t1, i, _i, a, aCount, _i2, b, t2, d, bCount, bin, maxCount;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                bucket = this.bucket, distance = this.distance, radius = this.radius;
                _this$options = this.options, tickMs = _this$options.tickMs, nIterationsBetweenTickChecks = _this$options.nIterationsBetweenTickChecks;
                strs = Object.keys(bucket);
                bins = [];
                strToBin = {}; // a => bin, only if buckets has >1 element

                t1 = new Date();
                i = 0;
                _i = 0;

              case 8:
                if (!(_i < strs.length)) {
                  _context.next = 38;
                  break;
                }

                a = strs[_i];
                aCount = bucket[a];
                _i2 = 0;

              case 12:
                if (!(_i2 < strs.length)) {
                  _context.next = 35;
                  break;
                }

                b = strs[_i2];
                i += 1;

                if (!((i & nIterationsBetweenTickChecks) === 0)) {
                  _context.next = 24;
                  break;
                }

                t2 = new Date();

                if (!(t2 - t1 >= tickMs)) {
                  _context.next = 24;
                  break;
                }

                this.progress = (i - 1) / (strs.length * strs.length);
                _context.next = 21;
                return tick();

              case 21:
                if (!this.canceled) {
                  _context.next = 23;
                  break;
                }

                throw new Error('canceled');

              case 23:
                t1 = new Date();

              case 24:
                if (!(a === b)) {
                  _context.next = 26;
                  break;
                }

                return _context.abrupt("continue", 32);

              case 26:
                if (!(a in strToBin && b in strToBin[a].bucket)) {
                  _context.next = 28;
                  break;
                }

                return _context.abrupt("continue", 32);

              case 28:
                if (!(b in strToBin && a in strToBin[b].bucket)) {
                  _context.next = 30;
                  break;
                }

                return _context.abrupt("continue", 32);

              case 30:
                d = distance(a, b);

                if (d <= radius) {
                  bCount = bucket[b];
                  bin = strToBin[a];

                  if (!bin) {
                    bin = {
                      name: a,
                      count: aCount,
                      bucket: _defineProperty({}, a, aCount)
                    };
                    strToBin[a] = bin;
                    bins.push(bin);
                  }

                  maxCount = bin.bucket[bin.name];

                  if (bCount > maxCount || bCount === maxCount && b.localeCompare(bin.name) < 0) {
                    bin.name = b;
                  }

                  bin.count += bCount;
                  bin.bucket[b] = bCount;
                }

              case 32:
                _i2++;
                _context.next = 12;
                break;

              case 35:
                _i++;
                _context.next = 8;
                break;

              case 38:
                this.progress = 1;
                return _context.abrupt("return", bins);

              case 40:
              case "end":
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function cluster() {
        return _cluster.apply(this, arguments);
      };
    }()
  }]);

  return KnnClusterer;
}();

function clusterByKnn(bucket, distance, radius) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  options = _objectSpread({
    tickMs: 8,
    nIterationsBetweenTickChecks: 0xfff
  }, options);
  return new KnnClusterer(bucket, distance, radius, options);
}

export { clusterByKey, clusterByKnn };
//# sourceMappingURL=index.js.map
